import { convertToSpec } from '@knapsack/spec-utils';
import { analyzeComponent } from './utils/react-component-analyzer.mjs';
export async function inferSpec({ path, namedExport, resolveFrom = process.cwd(), pkgPathAliases, }) {
    const { props: componentProps, slots: componentSlots } = await analyzeComponent({
        fileName: path,
        exportName: namedExport,
        pkgPathAliases,
        resolveFromDir: resolveFrom,
    });
    const props = {
        $schema: 'http://json-schema.org/draft-07/schema',
        type: 'object',
        required: [],
        properties: {},
    };
    const slots = componentSlots.reduce((acc, slot) => {
        if (!acc) {
            throw new Error('slots is undefined');
        }
        acc[slot.title] = slot;
        return acc;
    }, {});
    const extraDocs = [];
    // @TODO: Add overview table for react renderer
    // https://linear.app/knapsack/issue/KSP-5870/update-react-renderer-api-docs-to-include-overview
    // Path is currently the full path to the file, we need to get the package name from the path
    // // Add component overview
    // extraDocs.push({
    //   id: 'overview',
    //   title: 'Overview',
    //   description: '',
    //   tableRows: [
    //     {
    //       Title: 'Package',
    //       Description: `\`${path}\``,
    //     },
    //     {
    //       Title: 'Named Export',
    //       Description: `\`${namedExport}\``,
    //     },
    //   ],
    // });
    // NonNullable ensures we get the table row type from a defined extraDocs entry
    const propsTableRows = [];
    Object.entries(componentProps).forEach(([propName, propDetails]) => {
        var _a;
        const { required, typeInfo: propTypeInfo } = propDetails;
        if (propTypeInfo) {
            const { prop } = convertToSpec({
                typeInfo: propTypeInfo,
                trustRequiredProps: false,
            });
            if (prop) {
                props.properties[propName] = prop;
                if (propDetails.description) {
                    props.properties[propName].description = propDetails.description;
                }
                if (required) {
                    (_a = props.required) === null || _a === void 0 ? void 0 : _a.push(propName);
                }
            }
            propsTableRows.push({
                'Prop Name': `\`${propName}\``,
                Type: `\`${propTypeInfo.tsRawType || propTypeInfo.type || 'unknown'}\``,
                Required: required ? 'Yes' : 'No',
                Description: propDetails.description || '',
            });
        }
    });
    if (propsTableRows.length) {
        extraDocs.push({
            id: 'props',
            title: 'Props',
            description: '',
            tableRows: propsTableRows,
        });
    }
    if (Object.keys(slots || {}).length) {
        extraDocs.push({
            id: 'slots',
            title: 'Slots',
            description: '',
            tableRows: Object.entries(slots || {}).map(([slotName, slot]) => ({
                'Slot Name': `\`${slotName}\``,
                'Slot Title': slot.title,
            })),
        });
    }
    return {
        ksTemplateSpec: {
            isInferred: true,
            hidePropsTable: true,
            hideSlotsTable: true,
            extraDocs,
            props,
            slots,
        },
        props: componentProps,
        slots: componentSlots,
    };
}
//# sourceMappingURL=infer-spec.mjs.map