import { analyzeExport } from '@knapsack/spec-utils';
/**
 * Merges properties from multiple object types in a union
 */
const mergeUnionObjectProperties = ({ items, }) => {
    const mergedProperties = {};
    items.forEach((item) => {
        if (item.type === 'object' && item.properties) {
            Object.entries(item.properties).forEach(([propName, propInfo]) => {
                // If property already exists, merge with existing
                if (mergedProperties[propName]) {
                    mergedProperties[propName] = {
                        ...propInfo.typeInfo,
                        isOptional: mergedProperties[propName].isOptional ||
                            propInfo.typeInfo.isOptional,
                    };
                }
                else {
                    mergedProperties[propName] = propInfo.typeInfo;
                }
            });
        }
    });
    return mergedProperties;
};
/**
 * Checks if a TypeInfo represents a React node type or a function returning a React node
 */
const isReactNodeType = (typeInfo) => {
    var _a, _b, _c;
    // Check direct ReactNode types
    const typeName = (_c = (_b = (_a = typeInfo.tsMetadata) === null || _a === void 0 ? void 0 : _a.type) === null || _b === void 0 ? void 0 : _b.symbol) === null || _c === void 0 ? void 0 : _c.name;
    if (typeName === 'ReactNode' || typeInfo.tsRawType === 'ReactNode') {
        return true;
    }
    // Check if it's a function type that returns ReactNode
    if (typeInfo.type === 'function') {
        const { returnType } = typeInfo;
        if (returnType) {
            return isReactNodeType(returnType);
        }
    }
    // Handle union types
    if (typeInfo.type === 'union') {
        return typeInfo.items.some((item) => isReactNodeType(item));
    }
    return false;
};
/**
 * Processes prop information and adds it to the props object
 */
const processProps = ({ properties, }) => {
    const props = {};
    const slots = [];
    Object.entries(properties).forEach(([propName, propInfo]) => {
        var _a, _b;
        // Check for slot-like props first
        const isSlotProp = propName === 'children' || isReactNodeType(propInfo.typeInfo);
        if (isSlotProp) {
            slots.push({
                title: propName,
                ...(((_a = propInfo.jsDoc) === null || _a === void 0 ? void 0 : _a.description) && {
                    description: propInfo.jsDoc.description,
                }),
            });
        }
        else {
            // Only add to props if it's not a slot
            props[propName] = {
                required: !propInfo.typeInfo.isOptional,
                description: (_b = propInfo.jsDoc) === null || _b === void 0 ? void 0 : _b.description,
                typeInfo: propInfo.typeInfo,
            };
        }
    });
    return { props, slots };
};
/**
 * Analyzes a React component to extract its props and slots information
 */
export async function analyzeComponent({ fileName, exportName, pkgPathAliases, resolveFromDir, }) {
    var _a;
    const props = {};
    const slots = [];
    try {
        // Use analyzeExport to get TypeScript type information
        const { typeInfo } = analyzeExport({
            fileName,
            // if exportName (for example `template.alias`) is undefined, use 'default'
            exportName: exportName || 'default',
            pkgPathAliases,
            resolveFromDir,
        });
        // For function components, the first parameter contains the props type
        if (typeInfo.type === 'function') {
            const propsParam = (_a = typeInfo.parameters) === null || _a === void 0 ? void 0 : _a[0];
            if ((propsParam === null || propsParam === void 0 ? void 0 : propsParam.typeInfo.type) === 'union') {
                // Handle union component types by merging their properties
                const mergedProperties = mergeUnionObjectProperties({
                    items: propsParam.typeInfo.items,
                });
                // Wrap the properties in a typeInfo object
                const wrappedProperties = Object.fromEntries(Object.entries(mergedProperties).map(([key, info]) => [
                    key,
                    { typeInfo: info },
                ]));
                const result = processProps({
                    properties: wrappedProperties,
                });
                Object.assign(props, result.props);
                slots.push(...result.slots);
            }
            else if ((propsParam === null || propsParam === void 0 ? void 0 : propsParam.typeInfo.type) === 'object') {
                const result = processProps({
                    properties: propsParam.typeInfo.properties,
                });
                Object.assign(props, result.props);
                slots.push(...result.slots);
            }
        }
        if (typeInfo.type === 'class') {
            // @TODO: Implement class component analysis
            // https://linear.app/knapsack/issue/KSP-5879/setup-infer-spec-for-class-components-in-react-renderer
            // const propsParam = typeInfo.constructorParameters?.[0];
            throw new Error(`Class components are not setup for infer spec in react renderer`);
        }
    }
    catch (error) {
        // @TODO: Fallback to PropTypes
        // https://linear.app/knapsack/issue/KSP-5878/setup-infer-spec-for-prop-types-in-react-renderer
        // console.log(
        //   'Failed to analyze component types using TypeScript, falling back to PropTypes:\n\n',
        //   error.message,
        // );
        throw new Error(`Failed to infer spec for component "${exportName}" in  "${fileName}"`, {
            cause: error,
        });
    }
    return {
        props,
        slots,
    };
}
//# sourceMappingURL=react-component-analyzer.mjs.map