"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clearInferSpecCache = exports.demoAppName = void 0;
exports.getUsage = getUsage;
exports.getDemoAppUsage = getDemoAppUsage;
exports.getReactDocs = getReactDocs;
exports.copyReactAssets = copyReactAssets;
const app_1 = require("@knapsack/app");
const file_utils_1 = require("@knapsack/file-utils");
const utils_1 = require("@knapsack/utils");
const path_1 = __importDefault(require("path"));
const types_1 = require("@knapsack/types");
function renderUsageTemplate({ templateName, attributes, children, }) {
    return `
    <${templateName}
      ${attributes}
      ${children
        ? `>
        ${children}
      </${templateName}>`
        : '/>'}
  `;
}
exports.demoAppName = 'DemoApp';
function renderDemoAppTemplate({ imports = '', children, }) {
    return `${imports}

function ${exports.demoAppName}() {
  return (
    ${children}
  )
}
`;
}
async function getUsage(data) {
    const props = Object.keys(data.props || {}).map((key) => {
        const value = data.props[key];
        return {
            key,
            value,
        };
    });
    const { templateName, children, extraProps = [] } = data;
    if (!templateName) {
        throw new Error(`Cannot getUsage of a React Component when no "templateName" is provided.`);
    }
    const attributes = props.map(({ key, value }) => {
        switch (typeof value) {
            case 'string':
                // Is this a function like `() => {}` but not a "sizes" attribute for an image like `(min-width: 400px) 50vw, 100vw`?
                if (value.startsWith('(') && value.includes('=>')) {
                    return `${key}={${value}}`;
                }
                return `${key}="${value}"`;
            case 'boolean':
                return value ? `${key}` : `${key}={${value}}`;
            default:
                return `${key}={${JSON.stringify(value)}}`;
        }
    });
    const extraAttributes = [];
    const slotProps = {};
    extraProps.forEach(({ key, value }) => {
        var _a;
        slotProps[key] = (_a = slotProps[key]) !== null && _a !== void 0 ? _a : [];
        slotProps[key].push(value);
    });
    Object.entries(slotProps).forEach(([key, values]) => {
        const value = values.length === 1 ? values[0] : `<>${values.join('\n')}</>`;
        extraAttributes.push(`${key}={${value}}`);
    });
    const result = renderUsageTemplate({
        templateName,
        attributes: [...attributes, ...extraAttributes].join(' '),
        children,
    });
    return data.format
        ? (0, file_utils_1.formatCode)({
            contents: result,
            path: 'x.tsx', // doing this to set format language
        })
            .then((code) => code.trim())
            .catch(() => result.trim()) // if format fails, just return the unformatted code
        : result.trim();
}
async function getDemoAppUsage({ children, imports, format, }) {
    const code = renderDemoAppTemplate({
        children,
        imports,
    });
    if (!format)
        return code;
    return (0, file_utils_1.formatCode)({
        contents: code,
        path: 'x.tsx', // doing this to set format language
    });
}
async function getTypeScriptInfoFromFile({ src }) {
    const tsConfigPath = await (0, file_utils_1.findUp)('tsconfig.json', {
        cwd: path_1.default.dirname(src),
    });
    const rdTs = await import('react-docgen-typescript');
    const config = {
        shouldExtractLiteralValuesFromEnum: true,
        savePropValueAsString: true,
        skipChildrenPropWithoutDoc: false,
        // In addition to the ones listed here, which had not strangely included these below ~ https://github.com/styleguidist/react-docgen-typescript/blob/287e7012843cb26fed8f4bd8ee24e462c25a1414/src/parser.ts#L308
        customComponentTypes: ['VoidFunctionComponent', 'VFC'],
    };
    const parse = tsConfigPath
        ? rdTs.withCustomConfig(tsConfigPath, config).parse
        : rdTs.withDefaultConfig(config).parse;
    return {
        info: parse(src),
    };
}
function extractSpecFromTypeScriptInfo({ info: results, exportName, }) {
    try {
        if (!results)
            return false;
        const spec = {
            props: {
                $schema: 'http://json-schema.org/draft-07/schema',
                type: 'object',
                required: [],
                properties: {},
            },
            slots: {
            // children: {
            //   title: 'children',
            // },
            },
        };
        const isDefaultExport = !exportName || exportName === 'default';
        // if `isDefaultExport` then we just grab last result since `export default` TENDS to be last. that'll we can do really...
        const result = isDefaultExport
            ? results.at(-1)
            : results.find((r) => r.displayName === exportName);
        // log.inspect({ result, isDefaultExport, exportName }, 'ts result');
        if (!result)
            return false;
        const { displayName } = result;
        // log.inspect({ result }, 'result');
        Object.entries((result === null || result === void 0 ? void 0 : result.props) || {}).forEach(([propName, propDef]) => {
            var _a, _b, _c, _d, _e;
            const { name, description, defaultValue, required, type, parent } = propDef;
            // `type.name` can include `| undefined` if tsConfig has `strict: true`. So we need to handle these scenarios:
            // `string` or `string | undefined`
            // `'left' | 'right'` or `'left' | 'right' | undefined`
            const propType = (_a = type === null || type === void 0 ? void 0 : type.name) === null || _a === void 0 ? void 0 : _a.replace('| undefined', '').replace(/<.*/g, '').trim();
            switch (propType) {
                case 'string':
                    spec.props.properties[propName] = {
                        type: 'string',
                    };
                    break;
                case 'number':
                    spec.props.properties[propName] = {
                        type: 'number',
                    };
                    break;
                case 'boolean':
                case 'bool':
                    if (defaultValue && 'value' in defaultValue) {
                        // comes in as `'true'` - this turns it to `true`
                        defaultValue.value = defaultValue.value === 'true';
                    }
                    spec.props.properties[propName] = {
                        type: 'boolean',
                    };
                    break;
                case 'enum':
                    spec.props.properties[propName] = {
                        type: 'string',
                        // yes there is a double "value" & yes it is confusing
                        enum: [
                            // ensure list is unique
                            ...new Set(type.value.flatMap(({ value }) => {
                                if (!value)
                                    return [];
                                const item = (0, utils_1.removeWrappingQuotes)(value.trim());
                                if (item === 'undefined') {
                                    return [];
                                }
                                return [item];
                            })),
                        ],
                    };
                    break;
                case 'ReactNode':
                case 'React.ReactNode':
                case 'React.ReactElement':
                case 'ReactElement':
                    spec.slots[propName] = {
                        title: propName,
                    };
                    if (description) {
                        spec.slots[propName] = {
                            ...spec.slots[propName],
                            description,
                        };
                    }
                    break;
                case 'VFC':
                case 'FC':
                    spec.slots[propName] = {
                        title: propName,
                        description: description || 'A reference to a component',
                        isTemplateReference: true,
                        allowOnlyOne: true,
                        disallowText: true,
                    };
                    break;
                default: {
                    if ((_b = type === null || type === void 0 ? void 0 : type.name) === null || _b === void 0 ? void 0 : _b.startsWith('(')) {
                        // functions
                        spec.props.properties[propName] = {
                            // description: `\`${type.name}\` ${description}`,
                            typeof: 'function',
                            tsType: (_c = propDef === null || propDef === void 0 ? void 0 : propDef.type) === null || _c === void 0 ? void 0 : _c.name,
                        };
                    }
                    else if ((_d = type === null || type === void 0 ? void 0 : type.name) === null || _d === void 0 ? void 0 : _d.includes('|')) {
                        // enums / unions (multi-choice strings)
                        // comes in like this: `{ name: '"option1" | "option2" | "option3"' }`
                        const options = type.name.split('|').flatMap((enumItem) => {
                            if (!enumItem)
                                return [];
                            const item = (0, utils_1.removeWrappingQuotes)(enumItem.trim());
                            if (item === 'undefined') {
                                return [];
                            }
                            return [item];
                        });
                        if (options === null || options === void 0 ? void 0 : options.length) {
                            spec.props.properties[propName] = {
                                type: 'string',
                                // ensuring list is unique
                                enum: [...new Set(options)],
                            };
                        }
                    }
                    else {
                        spec.props.properties[propName] = {
                            tsType: (_e = propDef === null || propDef === void 0 ? void 0 : propDef.type) === null || _e === void 0 ? void 0 : _e.name,
                        };
                    }
                }
            }
            // assigning info that all would have
            if (spec.props.properties[propName]) {
                if (required)
                    spec.props.required.push(propName);
                if (description && !spec.props.properties[propName].description) {
                    spec.props.properties[propName].description = description;
                }
                if (defaultValue && 'value' in defaultValue) {
                    if ((0, types_1.isNumberProp)(spec.props.properties[propName])) {
                        spec.props.properties[propName].default = parseFloat(defaultValue.value);
                    }
                    else {
                        spec.props.properties[propName].default = (0, utils_1.removeWrappingQuotes)(defaultValue.value);
                    }
                }
            }
            // console.log('type', type);
        });
        return spec;
    }
    catch (error) {
        app_1.log.verbose('Could not infer spec from React TypeScript file', {
            exportName,
            error,
        }, 'react renderer');
        return false;
    }
}
async function getPropTypesInfoFromFile({ src }) {
    const reactDocs = await import('react-docgen');
    const { builtinResolvers } = reactDocs;
    const fileSrc = await (0, file_utils_1.readFile)(src);
    const { FindExportedDefinitionsResolver } = builtinResolvers;
    const results = reactDocs.parse(fileSrc, {
        resolver: new FindExportedDefinitionsResolver(),
        handlers: null,
        filename: src,
        // babelrc: false,
    });
    return {
        info: results,
    };
}
function extractSpecFromPropTypesInfo({ info: results, exportName, }) {
    try {
        const isDefaultExport = !exportName || exportName === 'default';
        // if `isDefaultExport` then we just grab last result since `export default` TENDS to be last. that'll we can do really...
        const result = isDefaultExport
            ? results.pop()
            : results.find((r) => r.displayName === exportName);
        // log.inspect(results, 'JSX PropTypes Results');
        // console.log(JSON.stringify(result, null, '  '));
        const spec = {
            isInferred: true,
            props: {
                $schema: 'http://json-schema.org/draft-07/schema',
                type: 'object',
                required: [],
                properties: {},
            },
            slots: {},
        };
        Object.entries((result === null || result === void 0 ? void 0 : result.props) || {}).forEach(([propName, propDef]) => {
            var _a;
            const { required, description, defaultValue } = propDef;
            switch ((_a = propDef === null || propDef === void 0 ? void 0 : propDef.type) === null || _a === void 0 ? void 0 : _a.name) {
                case 'string':
                    spec.props.properties[propName] = {
                        type: 'string',
                    };
                    break;
                case 'func':
                    spec.props.properties[propName] = {
                        type: 'string',
                    };
                    break;
                case 'bool':
                    spec.props.properties[propName] = {
                        type: 'boolean',
                    };
                    break;
                case 'node':
                    spec.slots[propName] = {
                        title: propName,
                        description,
                    };
            }
            // assigning info that all would have
            if (spec.props.properties[propName]) {
                if (required)
                    spec.props.required.push(propName);
                if (description && !spec.props.properties[propName].description) {
                    spec.props.properties[propName].description = description;
                }
                if (defaultValue && 'value' in defaultValue) {
                    spec.props.properties[propName].default =
                        typeof defaultValue.value === 'string'
                            ? (0, utils_1.removeWrappingQuotes)(defaultValue.value)
                            : defaultValue.value;
                }
            }
        });
        return spec;
    }
    catch (error) {
        app_1.log.verbose('Could not infer spec from React PropTypes', {
            exportName,
            error,
        }, 'react renderer');
        return false;
    }
}
function cleanUpSpec({ spec, }) {
    var _a;
    if (spec === false)
        return spec;
    // ok to mutate b/c spec was just created before passing in here
    Object.entries(((_a = spec === null || spec === void 0 ? void 0 : spec.props) === null || _a === void 0 ? void 0 : _a.properties) || {}).forEach(([propName, prop]) => {
        if ((0, types_1.isOptionsProp)(prop)) {
            // If the default is not in the enum, it should not be there
            // OR the enum is wrong
            if (!prop.enum.includes(prop.default)) {
                prop.default = undefined;
            }
        }
    });
    return spec;
}
async function getReactModuleInfoUncached({ src: unknownSrc, resolveFromDir, }) {
    const typesInfo = await (0, file_utils_1.resolvePath)({
        path: unknownSrc,
        resolveFromDir,
        resolveType: 'types',
    });
    if (typesInfo.exists) {
        const typeScriptInfo = await getTypeScriptInfoFromFile({
            src: typesInfo.absolutePath,
        });
        if (typeScriptInfo) {
            return {
                type: 'typescript',
                info: typeScriptInfo.info,
            };
        }
    }
    const jsInfo = await (0, file_utils_1.resolvePath)({
        path: unknownSrc,
        resolveFromDir,
    });
    if (!jsInfo.exists)
        return { type: 'unknown' };
    const { ext } = path_1.default.parse(jsInfo.absolutePath);
    switch (ext) {
        case '.jsx': {
            const propTypesInfo = await getPropTypesInfoFromFile({
                src: jsInfo.absolutePath,
            });
            return {
                type: 'propTypes',
                info: propTypesInfo.info,
            };
        }
        case '.ts':
        case '.tsx': {
            const typeScriptInfo = await getTypeScriptInfoFromFile({
                src: jsInfo.absolutePath,
            });
            return {
                type: 'typescript',
                info: typeScriptInfo.info,
            };
        }
        default:
            return {
                type: 'unknown',
            };
    }
}
const getReactModuleInfoCache = new Map();
const clearInferSpecCache = () => {
    app_1.log.verbose(`Clearing React TypeScript inferSpec cache...`);
    getReactModuleInfoCache.clear();
};
exports.clearInferSpecCache = clearInferSpecCache;
async function getReactModuleInfo(args) {
    const cacheKey = JSON.stringify(args);
    if (!getReactModuleInfoCache.has(cacheKey)) {
        getReactModuleInfoCache.set(cacheKey, getReactModuleInfoUncached(args));
    }
    return getReactModuleInfoCache.get(cacheKey);
}
async function getReactSpec({ src, exportName, resolveFromDir, }) {
    const reactModuleInfo = await getReactModuleInfo({
        src,
        resolveFromDir,
    });
    switch (reactModuleInfo.type) {
        case 'typescript':
            return extractSpecFromTypeScriptInfo({
                info: reactModuleInfo.info,
                exportName,
            });
        case 'propTypes':
            return extractSpecFromPropTypesInfo({
                info: reactModuleInfo.info,
                exportName,
            });
        case 'unknown':
        default:
            return false;
    }
}
async function getReactDocs(opt) {
    return cleanUpSpec({
        spec: await getReactSpec(opt),
    });
}
async function copyReactAssets(distDirAbsolute, publicPath) {
    try {
        const { version: reactVersion } = (0, file_utils_1.getModulePkgJson)('react');
        const { version: reactDomVersion } = (0, file_utils_1.getModulePkgJson)('react-dom');
        const [reactVersionMajorString] = reactVersion.split('.');
        const reactVersionMajor = parseInt(reactVersionMajorString, 10);
        if (reactVersionMajor === 19) {
            throw new Error(`React v19 is not supported yet. Please use v18 or lower.`);
        }
        // switch to resolving the root NPM package dir vs the entire file via require.resolve
        // workaround due to NPM packages using the exports field; prevents these from being auto-resolved
        const reactRoot = path_1.default.dirname(require.resolve('react', {
            paths: [process.cwd()],
        }));
        const reactDomRoot = path_1.default.dirname(require.resolve('react-dom', {
            paths: [process.cwd()],
        }));
        await Promise.all([
            (0, file_utils_1.copy)(path_1.default.join(reactRoot, 'umd/react.development.js'), path_1.default.join(distDirAbsolute, `react.development.${reactVersion}.js`)),
            (0, file_utils_1.copy)(path_1.default.join(reactRoot, 'umd/react.production.min.js'), path_1.default.join(distDirAbsolute, `react.production.min.${reactVersion}.js`)),
            (0, file_utils_1.copy)(path_1.default.join(reactDomRoot, 'umd/react-dom.production.min.js'), path_1.default.join(distDirAbsolute, `react-dom.production.min.${reactDomVersion}.js`)),
            (0, file_utils_1.copy)(path_1.default.join(reactDomRoot, 'umd/react-dom.development.js'), path_1.default.join(distDirAbsolute, `react-dom.development.${reactDomVersion}.js`)),
        ]);
        const reactFileSuffix = process.env.NODE_ENV === 'production' ? 'production.min' : 'development';
        return [
            path_1.default.join(publicPath, `react.${reactFileSuffix}.${reactVersion}.js`),
            path_1.default.join(publicPath, `react-dom.${reactFileSuffix}.${reactDomVersion}.js`),
        ];
    }
    catch (error) {
        app_1.log.warn('Error trying to copy "react" and "react-dom" JS files, are they installed? We want to use your exact versions.', error, 'templateRenderer:react');
        process.exit(1);
    }
}
//# sourceMappingURL=utils.js.map